use futures_util::FutureExt;
use tokio::sync::oneshot::{channel, Sender};

use crate::prelude::*;

pub mod http;
pub mod table;
pub mod telemetry;
#[cfg(test)]
pub mod testkit;

pub struct Terminator {
    signal: Sender<()>,
    completion: JoinHandle<Result<()>>,
}
impl Terminator {
    pub fn from_raw(signal: Sender<()>, completion: JoinHandle<Result<()>>) -> Terminator {
        Terminator { signal, completion }
    }

    pub fn new(completion: JoinHandle<Result<()>>) -> (Terminator, impl Future<Output = ()>) {
        let (signal, rx) = channel();
        (Terminator { signal, completion }, rx.map(|_| ()))
    }

    pub async fn terminate(self) -> Result<()> {
        match self.signal.send(()) {
            Ok(()) => {}
            Err(_) => log::error!("Termination channel is dropped"),
        }
        self.completion.await??;
        Ok(())
    }
}
